/*
 * sat_balancer - external squid ACL for load balancing.
 * Copyright (c) 2003 by Maxim Chirkov <mc@tyumen.ru>
 *
 */
 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>

/* #include <sys/types.h> */
/* #include <ctype.h> */

#define BUFSIZE 8192		/* the stdin buffer size */
#define DEF_SYNC_AFTER_REQ 100 /*         */
#define DEF_RAND_LENGHT 1000
#define DEF_RAND_OK 500

static void
usage()
{
    fprintf(stderr, "Usage: sat_balancer [-r sync_req_num] -f sync_file \n");
}

/*     . */
void update_rand_weight(char * syncfile_name, int *rand_lenght, int *rand_ok){
    FILE *FH;    
    char line[BUFSIZE+1];
    char *num_p1, *num_p2;

    if ((FH = fopen (syncfile_name, "r")) == NULL){
	fprintf(stderr, "Can't open sync file.\n");
	return;
    }

    if ((num_p1 = fgets (line, BUFSIZE, FH)) != NULL) {
	if ((num_p2 = strchr (line, '\n')) != NULL) {
	    *num_p2 = '\0';
	} else {
	    fprintf(stderr, "File format error: %s\n", line);
	    fclose(FH);    
	    return;
	}

	if ((num_p2 = strchr (line, '\t')) != NULL) {
	    *num_p2 = '\0';
	    num_p2++;
	} else {
	    fprintf(stderr, "File format error: %s\n", line);
	    fclose(FH);    
	    return;
	}
	*rand_ok = atoi(num_p1);
	*rand_lenght = atoi(num_p2);
    } else {
	    fprintf(stderr, "Zero file\n");
	    fclose(FH);    
	    return;
    }
    fclose(FH);    
    fprintf(stderr, "UPDATE Ok:%d / All:%d\n", *rand_ok, *rand_lenght);


}

/* /////////// MAIN //////////////// */
int main(int argc, char *argv[]){
    char buf[BUFSIZE+1];
    char *p;
    int ch;
    char *syncfile_name = NULL;
    struct stat f_stat;
    int max_sync_req = DEF_SYNC_AFTER_REQ;
    int rand_lenght = DEF_RAND_LENGHT;
    int rand_ok = DEF_RAND_OK;
    int sync_cycle = 0;
    time_t last_sync_time;
    

    /*       . */

    while ((ch = getopt(argc, argv, "f:r:")) != -1) {
	switch (ch) {
	case 'f':
	    syncfile_name = calloc(strlen(optarg) + 1, sizeof(char));
	    strcpy(syncfile_name, optarg);
	    break;
	case 'r':
	    max_sync_req = atoi(optarg);
	    break;
	default:
	    usage();
	    exit(1);
	}
    }
    
    if (optind < argc || syncfile_name == NULL){
	usage();
        exit(1);
    }
    if (stat(syncfile_name, &f_stat) == -1){
        fprintf(stderr, "ERROR: File not found (%s)\n", syncfile_name);
    	usage();
        exit(1);
    }

    /**************************************
     *     squid.
     */
    
    setvbuf(stdout, NULL, _IOLBF, 0);
    update_rand_weight(syncfile_name, &rand_lenght, &rand_ok);
    last_sync_time = time(NULL);
        
    while (fgets(buf, BUFSIZE, stdin)) {
	if ((p = strchr(buf, '\n')) != NULL) {
	    *p = '\0';
	} else {
	    /* too large message received.. skip and deny */
	    fprintf(stderr, "%s: ERROR: Too large: %s\n", argv[0], buf);
	    while (fgets(buf, BUFSIZE, stdin)) {
		fprintf(stderr, "%s: ERROR: Too large..: %s\n", argv[0], buf);
		if (strchr(buf, '\n') != NULL)
		    break;
	    }
	    goto error;
	}

	/*   ,   . */
	if (sync_cycle >= max_sync_req){
	    if (stat(syncfile_name, &f_stat) != -1 
		&& f_stat.st_mtime > last_sync_time){
		
		last_sync_time = time(NULL);
		sync_cycle = 0;
		update_rand_weight(syncfile_name, &rand_lenght, &rand_ok);
		
	    }
	}

	/*       . */
	if ((int) ((float) rand_lenght * rand()/(RAND_MAX + 1.0)) < rand_ok) {
	    printf("OK\n");
	} else {
error:
	    printf("ERR\n");
	}
	sync_cycle++;
    }
    return 0;
}


